"""\
This file is an installation script for CMFPlone (ZPT skins).  It's meant
to be used as an External Method.  To use, add an external method to the
root of the CMF Site that you want ZPT skins registered in with the
configuration:

 id: install_plone
 title: Install Decor Skins *optional*
 module name: CMFPlone.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Test' tab.  The install function will execute and give
information about the steps it took to register and install the
ZPT skins into the CMF Site instance. 
"""

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import cmfplone_globals

from Acquisition import aq_base
from cStringIO import StringIO
import string
import string 

from Acquisition import Implicit
import Persistence

def changeImmediateViews(self, outStream):
    """ the editing process of CMF is akward, it expects you to fill in metadata before you fill in
        content information.  according to The Plone Way(tm) we create object, edit the object, set
        Properties.  
    """
    typesToSkip=['Folder', 'Discussion Item', 'Topic']
    typesTool=getToolByName(self, 'portal_types')
    outStream.write('Plone is now setting the immediate_view to the object\'s edit view\n')
    for contentType in typesTool.listContentTypes():        
        try:
            typeInfo=typesTool.getTypeInfo(contentType)
            if typeInfo.Type() not in typesToSkip:
                typeObj=getattr(typesTool, typeInfo.getId())
                view=typeInfo.getActionById('edit')
                typeObj._setPropValue('immediate_view', view)
                outStream.write(typeInfo.Type() + " has had its immediate view changed to " + view + '\n')
            
            if typeInfo.Type()=='Folder':
                typeObj=getattr(typesTool, typeInfo.getId())
                view='folder_contents'
                typeObj._setPropValue('immediate_view', view)
                outStream.write(typeInfo.Type() + " has had its immediate view changed to " + view + '\n')
        
        except: 
            pass #gulp!

def checkDependencies(self, outStream):
    """ attempt to report back any dependncies"""

    rootObj=self.aq_parent.restrictedTraverse( ('',) )
    control_panel = getattr(rootObj, 'Control_Panel', None)
    if control_panel:
        products=control_panel.Products
        if not hasattr(products, 'DCWorkflow'):
            outStream.write('You do not have DCWorkflow installed.  it is *highly* recommended you install it.\n')
        else:
            wf_tool=getToolByName(self, 'portal_workflow')
            if 'plone_workflow' not in wf_tool.objectIds():
                wf_tool.manage_addWorkflow(id='plone_workflow', workflow_type='default_workflow (Web-configurable workflow [Revision 2])')            
                wf_tool.setDefaultChain('plone_workflow')            
                wf_tool.updateRoleMappings()
                outStream.write('Created plone_workflow in portal_workflow tool and set it to default.\n')        
    else:
        outStream.write('could not access Root object.\n')

    if hasattr(rootObj, 'portal_membership'): #default membership homepages dont participate in workflow
        rootObj.manage_delObjects('portal_membership') 
        rootObj.manage_addProduct['CMFPlone'].manage_addTool(type='Default Membership Tool')

    outStream.write('over-wrote createMemberarea method on membership_tool, so homepage creation participates in workflow')

def install_SubSkin(self, outStream, skinName, skinFolder):
    """ Installs a subskin, should be just 1 folder that overrides the needed plone /img and stylesheet
        i.e. skinName=Plone IE5.5, skinFolder=plone_ie55
    """
    skinstool=getToolByName(self, 'portal_skins')
    path = skinstool.getSkinPath('Plone Default') #default
    if not path:
        outStream.write('Plone skin not found, aborting\n')
        return
    path = map( string.strip, string.split( path,',' ) )
    try:
        path.insert( path.index( 'custom')+1, skinFolder )
    except ValueError:
        path.append(skinFolder)
    path = string.join( path, ', ' )
    skinstool.addSkinSelection( skinName, path )


def install(self):
    " Register the Plone Skins with portal_skins and friends "
    out = StringIO()
    skinstool = getToolByName(self, 'portal_skins')

    #   check for dependencies
    checkDependencies( self, out)
    
    addDirectoryViews( skinstool, 'skins', cmfplone_globals )
    out.write( "Added CMFPlone directory views to portal_skins\n" )
    #
    #   Add a new skin, 'Plone', copying 'ZPT', if it exists, and then
    #   add our directories only to it.
    #
    if skinstool.getSkinPath( 'Plone' ) is None:
        path = skinstool.getSkinPath( skinstool.getDefaultSkin() )        
        if not path:
            try:
                path = skinstool.getSkinPath( 'ZPT' )
            except:
                path = skinstool.getSkinPath( 'Basic' ) #new CMF that doesnt use CMFDecor

        path = map( string.strip, string.split( path,',' ) )
        for plonedir in ( 'plone_content'
                      , 'plone_custom'
                      , 'plone_images'
                      , 'plone_forms'
                      , 'plone_scripts'
                      , 'plone_styles'
                      , 'plone_templates'
                      , 'plone_3rdParty'
                      ):
            try:
                path.insert( path.index( 'custom')+1, plonedir )
            except ValueError:
                path.append( plonedir )
                    
        path = string.join( path, ', ' )
        skinstool.addSkinSelection( 'Plone Default', path )
        out.write( "Added Plone Default skin\n" )

        install_SubSkin(self, out, 'Plone DHTML', 'plone_ie55')
        out.write('Plone DHTML subskin successfully installed\n')
        
        install_SubSkin(self, out, 'Plone Mozilla', 'plone_mozilla')
        out.write('Mozilla subskin successfully installed\n')

        changeImmediateViews(self, out)
        out.write( "\n\n\n!!Extra Configuration Done!!\n" )
        
        skinstool.default_skin='Plone Default'
        out.write( "The Plone skin has now been set as this portals default skin\n" )
        
        MembershipTool=getToolByName(self, 'portal_membership')
        if not MembershipTool.getMemberareaCreationFlag():
            MembershipTool.setMemberareaCreationFlag()
            out.write( "Memberarea Creation turned on - existing logins will have folders created upon login.\n" )

        skinstool.allow_any=1 #allow people to arbitrarily select skins
        
    else:
        out.write( "Plone skin already exists. Remove before installing again.\n" )

    return out.getvalue()
